@@ -1,11 +1,13 @@
 from vrcpy import Client, AClient, objects, aobjects
 from vrcpy.errors import WebSocketError, WebSocketOpenedError, IntegretyError
+from vrcpy.types import State
 import threading
 import websocket
 import asyncio
 import time
 import json
-import ssl
+# import ssl
+
 
 class _WSSClient:
     def _do_function(self, function, *args):
@@ -31,7 +33,8 @@ class _WSSClient:
         if message["type"] in switch:
             self._do_function(switch[message["type"]], json.loads(message["content"]))
         else:
-            self._do_function(self._ws_unhandled_event, message["type"], json.loads(message["content"]))
+            self._do_function(self._ws_unhandled_event,
+                              message["type"], json.loads(message["content"]))
 
     def _ws_error(self, ws, error):
         raise WebSocketError(error)
@@ -62,7 +65,7 @@ class _WSSClient:
         else:
             auth = self.api.session.cookies.get("auth")
 
-        #websocket.enableTrace(True)
+        # websocket.enableTrace(True)
         self.ws = websocket.WebSocketApp(
             "wss://pipeline.vrchat.cloud/?authToken="+auth,
             on_message=self._ws_message,
@@ -81,6 +84,57 @@ class _WSSClient:
         self.reconnect = False
         self.ws.close()
 
+    # Utility
+
+    def _update_friend(self, newUser, id):
+        # Updates self.me friends lists
+        # Returns old user
+
+        oldUser = None
+
+        for user in self.me.onlineFriends:
+            if user.id == id:
+                oldUser = user
+                self.me.onlineFriends.remove(user)
+                break
+
+        if oldUser is None:
+            for user in self.me.offlineFriends:
+                if user.id == id:
+                    oldUser = user
+                    self.me.offlineFriends.remove(user)
+                    break
+
+        if newUser is not None:
+            if newUser.state is State.Offline:
+                self.me.offlineFriends.append(newUser)
+            else:
+                self.me.onlineFriends.append(newUser)
+
+        self.me.friends = self.me.offlineFriends + self.me.onlineFriends
+        return oldUser
+
+    # WS Event decorator
+
+    def event(self, func):
+        """Decorator that overwrites class ws event hooks
+
+        Example
+        ---------
+
+        @client.event
+        def on_connect():
+            print("Connected to wss pipeline.")
+
+        """
+
+        if func.__name__.startswith("on_") and hasattr(self, func.__name__):
+            setattr(self, func.__name__, func)
+            return func
+        else:
+            raise TypeError("Registered event must be a valid event function")
+
+
 class WSSClient(Client, _WSSClient):
     # User WS overwrites
 
@@ -120,10 +174,14 @@ class WSSClient(Client, _WSSClient):
     # WS handles
 
     def _ws_friend_online(self, content):
-        self.on_friend_online(objects.User(self, content["user"]))
+        user = objects.User(self, content["user"])
+        self._update_friend(user, user.id)
+        self.on_friend_online(user)
 
     def _ws_friend_active(self, content):
-        self.on_friend_active(objects.User(self, content["user"]))
+        user = objects.User(self, content["user"])
+        self._update_friend(user, user.id)
+        self.on_friend_active(user)
 
     def _ws_friend_location(self, content):
         user = objects.User(self, content["user"])
@@ -140,19 +198,27 @@ class WSSClient(Client, _WSSClient):
         instance = world.fetch_instance(content["instance"])
         location = objects.Location(self, content["location"])
 
+        self._update_friend(user, user.id)
         self.on_friend_location(user, world, location, instance)
 
     def _ws_friend_offline(self, content):
-        self.on_friend_offline(self.fetch_user_by_id(content["userId"]))
+        user = self.fetch_user_by_id(content["userId"])
+        self._update_friend(user, user.id)
+        self.on_friend_offline(user)
 
     def _ws_friend_add(self, content):
-        self.on_friend_add(objects.User(self, content["user"]))
+        user = objects.User(self, content["user"])
+        self._update_friend(user, user.id)
+        self.on_friend_add(user)
 
     def _ws_friend_delete(self, content):
-        self.on_friend_delete(self.fetch_user_by_id(content["userId"]))
+        user = self._update_friend(None, content["userId"])
+        self.on_friend_delete(user)
 
     def _ws_friend_update(self, content):
-        self.on_friend_update(objects.User(self, content["user"]))
+        user = objects.User(self, content["user"])
+        self._update_friend(user, user.id)
+        self.on_friend_update(user)
 
     def _ws_notification(self, content):
         self.on_notification(objects.Notification(self, content))
@@ -164,21 +230,24 @@ class WSSClient(Client, _WSSClient):
 
     def login(self, username, password):
         super().login(username, password)
-        if self.loggedIn: self.connect()
+        if self.loggedIn:
+            self.connect()
 
     def login2fa(self, username, password, code=None, verify=False):
         super().login2fa(username, password, code, verify)
 
     def verify2fa(self, code):
         super().verify2fa(code)
-        if self.loggedIn: self.connect()
+        if self.loggedIn:
+            self.connect()
 
     def logout(self):
         super().logout()
-        if not self.loggedIn: self.disconnect()
+        if not self.loggedIn:
+            self.disconnect()
 
     def __init__(self, verify=True, reconnect=True):
-        super().__init__(verify, True) # Caching is always true for ws client
+        super().__init__(verify, True)  # Caching is always true for ws client
 
         self.ws = None
         self._wsthread = None
@@ -186,6 +255,7 @@ class WSSClient(Client, _WSSClient):
 
         self.reconnect = reconnect
 
+
 class AWSSClient(AClient, _WSSClient):
     # User WS overwrites
 
@@ -225,10 +295,14 @@ class AWSSClient(AClient, _WSSClient):
     # WS handles
 
     async def _ws_friend_online(self, content):
-        await self.on_friend_online(aobjects.User(self, content["user"]))
+        user = aobjects.User(self, content["user"])
+        self._update_friend(user, user.id)
+        await self.on_friend_online(user)
 
     async def _ws_friend_active(self, content):
-        await self.on_friend_active(aobjects.User(self, content["user"]))
+        user = aobjects.User(self, content["user"])
+        self._update_friend(user, user.id)
+        await self.on_friend_active(user)
 
     async def _ws_friend_location(self, content):
         user = aobjects.User(self, content["user"])
@@ -245,19 +319,27 @@ class AWSSClient(AClient, _WSSClient):
         instance = await world.fetch_instance(content["instance"])
         location = aobjects.Location(self, content["location"])
 
+        self._update_friend(user, user.id)
         await self.on_friend_location(user, world, location, instance)
 
     async def _ws_friend_offline(self, content):
-        await self.on_friend_offline(await self.fetch_user_by_id(content["userId"]))
+        user = await self.fetch_user_by_id(content["userId"])
+        self._update_friend(user, user.id)
+        await self.on_friend_offline(user)
 
     async def _ws_friend_add(self, content):
-        await self.on_friend_add(aobjects.User(self, content["user"]))
+        user = aobjects.User(self, content["user"])
+        self._update_friend(user, user.id)
+        await self.on_friend_add(user)
 
     async def _ws_friend_delete(self, content):
-        await self.on_friend_delete(await self.fetch_user_by_id(content["userId"]))
+        user = self._update_friend(None, content["userId"])
+        await self.on_friend_delete(user)
 
     async def _ws_friend_update(self, content):
-        await self.on_friend_update(aobjects.User(self, content["user"]))
+        user = aobjects.User(self, content["user"])
+        self._update_friend(user, user.id)
+        await self.on_friend_update(user)
 
     async def _ws_notification(self, content):
         await self.on_notification(aobjects.Notification(self, content))
@@ -269,21 +351,24 @@ class AWSSClient(AClient, _WSSClient):
 
     async def login(self, username, password):
         await super().login(username, password)
-        if self.loggedIn: self.connect()
+        if self.loggedIn:
+            self.connect()
 
     async def login2fa(self, username, password, code=None, verify=False):
         await super().login2fa(username, password, code, verify)
 
     async def verify2fa(self, code):
         await super().verify2fa(code)
-        if self.loggedIn: self.connect()
+        if self.loggedIn:
+            self.connect()
 
     async def logout(self):
         await super().logout()
-        if not self.loggedIn: self.disconnect()
+        if not self.loggedIn:
+            self.disconnect()
 
     def __init__(self, verify=True, reconnect=True):
-        super().__init__(verify, True) # Caching is always true for ws client
+        super().__init__(verify, True)  # Caching is always true for ws client
 
         self.ws = None
         self._wsthread = None
